"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.registerWebhook = registerWebhook;
exports.ApiVersion = void 0;

var _network = require("@shopify/network");

var _types = require("./types");

let ApiVersion;
exports.ApiVersion = ApiVersion;

(function (ApiVersion) {
  ApiVersion["April19"] = "2019-04";
  ApiVersion["July19"] = "2019-07";
  ApiVersion["October19"] = "2019-10";
  ApiVersion["January20"] = "2020-01";
  ApiVersion["April20"] = "2020-04";
  ApiVersion["Unstable"] = "unstable";
  ApiVersion["Unversioned"] = "unversioned";
})(ApiVersion || (exports.ApiVersion = ApiVersion = {}));

async function registerWebhook({
  address,
  topic,
  accessToken,
  shop,
  apiVersion
}) {
  const response = await fetch(`https://${shop}/admin/api/${apiVersion}/graphql.json`, {
    method: _network.Method.Post,
    body: buildQuery(topic, address),
    headers: {
      [_types.WebhookHeader.AccessToken]: accessToken,
      [_network.Header.ContentType]: 'application/graphql'
    }
  });
  const result = await response.json();

  if (result.data && result.data.webhookSubscriptionCreate && result.data.webhookSubscriptionCreate.webhookSubscription) {
    return {
      success: true,
      result
    };
  } else {
    return {
      success: false,
      result
    };
  }
}

function buildQuery(topic, callbackUrl) {
  return `
    mutation webhookSubscriptionCreate {
      webhookSubscriptionCreate(topic: ${topic}, webhookSubscription: {callbackUrl: "${callbackUrl}"}) {
        userErrors {
          field
          message
        }
        webhookSubscription {
          id
        }
      }
    }
  `;
}